@@ -18,29 +18,29 @@ package org.apache.commons.io.input;
 
 import static org.junit.jupiter.api.Assertions.assertEquals;
 import static org.junit.jupiter.api.Assertions.assertFalse;
+import static org.junit.jupiter.api.Assertions.assertThrows;
 import static org.junit.jupiter.api.Assertions.assertTrue;
 
 import java.io.ByteArrayInputStream;
 import java.io.IOException;
-import java.io.InputStream;
 
 import org.junit.jupiter.api.BeforeEach;
 import org.junit.jupiter.api.Test;
 
 /**
- * JUnit Test Case for {@link AutoCloseInputStream}.
+ * Tests {@link AutoCloseInputStream}.
  */
 public class AutoCloseInputStreamTest {
 
     private byte[] data;
 
-    private InputStream stream;
+    private AutoCloseInputStream stream;
 
     private boolean closed;
 
     @BeforeEach
     public void setUp() {
-        data = new byte[] {'x', 'y', 'z'};
+        data = new byte[] { 'x', 'y', 'z' };
         stream = new AutoCloseInputStream(new ByteArrayInputStream(data) {
             @Override
             public void close() {
@@ -50,6 +50,12 @@ public class AutoCloseInputStreamTest {
         closed = false;
     }
 
+    @Test
+    public void testBuilderGet() {
+        // java.lang.IllegalStateException: origin == null
+        assertThrows(IllegalStateException.class, () -> AutoCloseInputStream.builder().get());
+    }
+
     @Test
     public void testClose() throws IOException {
         stream.close();
@@ -57,6 +63,13 @@ public class AutoCloseInputStreamTest {
         assertEquals(-1, stream.read(), "read()");
     }
 
+    @Test
+    public void testFinalize() throws Throwable {
+        stream.finalize();
+        assertTrue(closed, "closed");
+        assertEquals(-1, stream.read(), "read()");
+    }
+
     @Test
     public void testRead() throws IOException {
         for (final byte element : data) {
@@ -99,10 +112,7 @@ public class AutoCloseInputStreamTest {
         assertEquals(-1, stream.read(b, 0, b.length), "read(b, off, len)");
     }
 
-    @Test
-    public void testResetBeforeEnd() throws IOException {
-        final String inputStr = "1234";
-        final AutoCloseInputStream inputStream = new AutoCloseInputStream(new ByteArrayInputStream(inputStr.getBytes()));
+    private void testResetBeforeEnd(final AutoCloseInputStream inputStream) throws IOException {
         inputStream.mark(1);
         assertEquals('1', inputStream.read());
         inputStream.reset();
@@ -121,4 +131,32 @@ public class AutoCloseInputStreamTest {
         assertEquals('1', inputStream.read());
     }
 
+    @Test
+    public void testResetBeforeEndCtor() throws IOException {
+        try (final AutoCloseInputStream inputStream = new AutoCloseInputStream(new ByteArrayInputStream("1234".getBytes()))) {
+            testResetBeforeEnd(inputStream);
+        }
+    }
+
+    @Test
+    public void testResetBeforeEndSetByteArray() throws IOException {
+        try (final AutoCloseInputStream inputStream = AutoCloseInputStream.builder().setByteArray("1234".getBytes()).get()) {
+            testResetBeforeEnd(inputStream);
+        }
+    }
+
+    @Test
+    public void testResetBeforeEndSetCharSequence() throws IOException {
+        try (final AutoCloseInputStream inputStream = AutoCloseInputStream.builder().setCharSequence("1234").get()) {
+            testResetBeforeEnd(inputStream);
+        }
+    }
+
+    @Test
+    public void testResetBeforeEndSetInputStream() throws IOException {
+        try (final AutoCloseInputStream inputStream = AutoCloseInputStream.builder().setInputStream(new ByteArrayInputStream("1234".getBytes())).get()) {
+            testResetBeforeEnd(inputStream);
+        }
+    }
+
 }
