@@ -20,8 +20,8 @@ import static org.junit.jupiter.api.Assertions.assertEquals;
 import static org.junit.jupiter.api.Assertions.assertFalse;
 import static org.junit.jupiter.api.Assertions.assertNotNull;
 import static org.junit.jupiter.api.Assertions.assertNull;
+import static org.junit.jupiter.api.Assertions.assertThrows;
 import static org.junit.jupiter.api.Assertions.assertTrue;
-import static org.junit.jupiter.api.Assertions.fail;
 import static org.junit.jupiter.api.Assumptions.assumeFalse;
 import static org.junit.jupiter.api.Assumptions.assumeTrue;
 
@@ -45,18 +45,14 @@ import org.xml.sax.SAXParseException;
 
 /**
  * Test case for {@link BOMInputStream}.
- *
  */
 @SuppressWarnings("ResultOfMethodCallIgnored")
 public class BOMInputStreamTest {
-    //----------------------------------------------------------------------------
-    //  Support code
-    //----------------------------------------------------------------------------
 
     /**
      *  A mock InputStream that expects {@code close()} to be called.
      */
-    private static class ExpectCloseInputStream extends InputStream {
+    private static final class ExpectCloseInputStream extends InputStream {
         private boolean _closeCalled;
 
         public void assertCloseCalled() {
@@ -144,7 +140,7 @@ public class BOMInputStreamTest {
     /**
      *  Creates the underlying data stream, with or without BOM.
      */
-    private InputStream createUtf8DataStream(final byte[] baseData, final boolean addBOM) {
+    private InputStream createUtf8Input(final byte[] baseData, final boolean addBOM) {
         byte[] data = baseData;
         if (addBOM) {
             data = new byte[baseData.length + 3];
@@ -156,9 +152,24 @@ public class BOMInputStreamTest {
         return new ByteArrayInputStream(data);
     }
 
-    //----------------------------------------------------------------------------
-    //  Test cases
-    //----------------------------------------------------------------------------
+    private boolean doesSaxSupportCharacterSet(final String charsetName) throws ParserConfigurationException, SAXException, IOException {
+        final DocumentBuilder documentBuilder = DocumentBuilderFactory.newInstance().newDocumentBuilder();
+        try (InputStream byteStream = CharSequenceInputStream.builder().setCharSequence("<?xml version=\"1.0\" encoding=\"" + charsetName + "\"?><Z/>")
+                .setCharset(charsetName).get()) {
+            final InputSource is = new InputSource(byteStream);
+            is.setEncoding(charsetName);
+            documentBuilder.parse(is);
+        } catch (final SAXParseException e) {
+            if (e.getMessage().contains(charsetName)) {
+                return false;
+            }
+        }
+        return true;
+    }
+
+    private boolean jvmAndSaxBothSupportCharset(final String charSetName) throws ParserConfigurationException, SAXException, IOException {
+        return Charset.isSupported(charSetName) &&  doesSaxSupportCharacterSet(charSetName);
+    }
 
     private void parseXml(final InputStream in) throws SAXException, IOException, ParserConfigurationException {
         final Document doc = DocumentBuilderFactory.newInstance().newDocumentBuilder().parse(new InputSource(in));
@@ -173,16 +184,17 @@ public class BOMInputStreamTest {
     }
 
     private void readBOMInputStreamTwice(final String resource) throws Exception {
-        final InputStream inputStream = this.getClass().getResourceAsStream(resource);
-        assertNotNull(inputStream);
-        final BOMInputStream bomInputStream = new BOMInputStream(inputStream);
-        bomInputStream.mark(1000000);
+        try (InputStream inputStream = this.getClass().getResourceAsStream(resource)) {
+            assertNotNull(inputStream);
+            try (BOMInputStream bomInputStream = BOMInputStream.builder().setInputStream(inputStream).get()) {
+                bomInputStream.mark(1_000_000);
 
-        this.readFile(bomInputStream);
-        bomInputStream.reset();
-        this.readFile(bomInputStream);
-        inputStream.close();
-        bomInputStream.close();
+                this.readFile(bomInputStream);
+                bomInputStream.reset();
+                this.readFile(bomInputStream);
+                inputStream.close();
+            }
+        }
     }
 
     private void readFile(final BOMInputStream bomInputStream) throws Exception {
@@ -196,7 +208,7 @@ public class BOMInputStreamTest {
     @Test
     public void testAvailableWithBOM() throws Exception {
         final byte[] data = { 'A', 'B', 'C', 'D' };
-        try (final InputStream in = new BOMInputStream(createUtf8DataStream(data, true))) {
+        try (InputStream in = BOMInputStream.builder().setInputStream(createUtf8Input(data, true)).get()) {
             assertEquals(7, in.available());
         }
     }
@@ -204,16 +216,22 @@ public class BOMInputStreamTest {
     @Test
     public void testAvailableWithoutBOM() throws Exception {
         final byte[] data = { 'A', 'B', 'C', 'D' };
-        try (final InputStream in = new BOMInputStream(createUtf8DataStream(data, false))) {
+        try (InputStream in = BOMInputStream.builder().setInputStream(createUtf8Input(data, false)).get()) {
             assertEquals(4, in.available());
         }
     }
 
+    @Test
+    public void testBuilderGet() {
+        // java.lang.IllegalStateException: origin == null
+        assertThrows(IllegalStateException.class, () -> BOMInputStream.builder().get());
+    }
+
     @Test
     // this is here for coverage
     public void testClose() throws Exception {
-        try (final ExpectCloseInputStream del = new ExpectCloseInputStream()) {
-            try (final InputStream in = new BOMInputStream(del)) {
+        try (ExpectCloseInputStream del = new ExpectCloseInputStream()) {
+            try (InputStream in = new BOMInputStream(del)) {
                 // nothing
             }
             del.assertCloseCalled();
@@ -223,7 +241,7 @@ public class BOMInputStreamTest {
     @Test
     public void testEmptyBufferWithBOM() throws Exception {
         final byte[] data = {};
-        try (final InputStream in = new BOMInputStream(createUtf8DataStream(data, true))) {
+        try (InputStream in = BOMInputStream.builder().setInputStream(createUtf8Input(data, true)).get()) {
             final byte[] buf = new byte[1024];
             assertEquals(-1, in.read(buf));
         }
@@ -232,7 +250,7 @@ public class BOMInputStreamTest {
     @Test
     public void testEmptyBufferWithoutBOM() throws Exception {
         final byte[] data = {};
-        try (final InputStream in = new BOMInputStream(createUtf8DataStream(data, false))) {
+        try (InputStream in = BOMInputStream.builder().setInputStream(createUtf8Input(data, false)).get()) {
             final byte[] buf = new byte[1024];
             assertEquals(-1, in.read(buf));
         }
@@ -241,7 +259,7 @@ public class BOMInputStreamTest {
     @Test
     public void testGetBOMFirstThenRead() throws Exception {
         final byte[] data = { 'A', 'B', 'C' };
-        try (final BOMInputStream in = new BOMInputStream(createUtf8DataStream(data, true))) {
+        try (BOMInputStream in = BOMInputStream.builder().setInputStream(createUtf8Input(data, true)).get()) {
             assertEquals(ByteOrderMark.UTF_8, in.getBOM(), "getBOM");
             assertTrue(in.hasBOM(), "hasBOM()");
             assertTrue(in.hasBOM(ByteOrderMark.UTF_8), "hasBOM(UTF-8)");
@@ -255,7 +273,7 @@ public class BOMInputStreamTest {
     @Test
     public void testGetBOMFirstThenReadInclude() throws Exception {
         final byte[] data = { 'A', 'B', 'C' };
-        try (final BOMInputStream in = new BOMInputStream(createUtf8DataStream(data, true), true)) {
+        try (BOMInputStream in = BOMInputStream.builder().setInputStream(createUtf8Input(data, true)).setInclude(true).get()) {
             assertTrue(in.hasBOM(), "hasBOM()");
             assertTrue(in.hasBOM(ByteOrderMark.UTF_8), "hasBOM(UTF-8)");
             assertEquals(ByteOrderMark.UTF_8, in.getBOM(), "getBOM");
@@ -272,7 +290,7 @@ public class BOMInputStreamTest {
     @Test
     public void testLargeBufferWithBOM() throws Exception {
         final byte[] data = { 'A', 'B', 'C' };
-        try (final InputStream in = new BOMInputStream(createUtf8DataStream(data, true))) {
+        try (InputStream in = BOMInputStream.builder().setInputStream(createUtf8Input(data, true)).get()) {
             final byte[] buf = new byte[1024];
             assertData(data, buf, in.read(buf));
         }
@@ -281,7 +299,7 @@ public class BOMInputStreamTest {
     @Test
     public void testLargeBufferWithoutBOM() throws Exception {
         final byte[] data = { 'A', 'B', 'C' };
-        try (final InputStream in = new BOMInputStream(createUtf8DataStream(data, false))) {
+        try (InputStream in = BOMInputStream.builder().setInputStream(createUtf8Input(data, false)).get()) {
             final byte[] buf = new byte[1024];
             assertData(data, buf, in.read(buf));
         }
@@ -290,7 +308,7 @@ public class BOMInputStreamTest {
     @Test
     public void testLeadingNonBOMBufferedRead() throws Exception {
         final byte[] data = { (byte) 0xEF, (byte) 0xAB, (byte) 0xCD };
-        try (final InputStream in = new BOMInputStream(createUtf8DataStream(data, false))) {
+        try (InputStream in = BOMInputStream.builder().setInputStream(createUtf8Input(data, false)).get()) {
             final byte[] buf = new byte[1024];
             assertData(data, buf, in.read(buf));
         }
@@ -299,7 +317,7 @@ public class BOMInputStreamTest {
     @Test
     public void testLeadingNonBOMSingleRead() throws Exception {
         final byte[] data = { (byte) 0xEF, (byte) 0xAB, (byte) 0xCD };
-        try (final InputStream in = new BOMInputStream(createUtf8DataStream(data, false))) {
+        try (InputStream in = BOMInputStream.builder().setInputStream(createUtf8Input(data, false)).get()) {
             assertEquals(0xEF, in.read());
             assertEquals(0xAB, in.read());
             assertEquals(0xCD, in.read());
@@ -310,7 +328,7 @@ public class BOMInputStreamTest {
     @Test
     public void testMarkResetAfterReadWithBOM() throws Exception {
         final byte[] data = { 'A', 'B', 'C', 'D' };
-        try (final InputStream in = new BOMInputStream(createUtf8DataStream(data, true))) {
+        try (InputStream in = BOMInputStream.builder().setInputStream(createUtf8Input(data, true)).get()) {
             assertTrue(in.markSupported());
 
             in.read();
@@ -326,7 +344,7 @@ public class BOMInputStreamTest {
     @Test
     public void testMarkResetAfterReadWithoutBOM() throws Exception {
         final byte[] data = { 'A', 'B', 'C', 'D' };
-        try (final InputStream in = new BOMInputStream(createUtf8DataStream(data, false))) {
+        try (InputStream in = BOMInputStream.builder().setInputStream(createUtf8Input(data, false)).get()) {
             assertTrue(in.markSupported());
 
             in.read();
@@ -342,7 +360,7 @@ public class BOMInputStreamTest {
     @Test
     public void testMarkResetBeforeReadWithBOM() throws Exception {
         final byte[] data = { 'A', 'B', 'C', 'D' };
-        try (final InputStream in = new BOMInputStream(createUtf8DataStream(data, true))) {
+        try (InputStream in = BOMInputStream.builder().setInputStream(createUtf8Input(data, true)).get()) {
             assertTrue(in.markSupported());
 
             in.mark(10);
@@ -357,7 +375,7 @@ public class BOMInputStreamTest {
     @Test
     public void testMarkResetBeforeReadWithoutBOM() throws Exception {
         final byte[] data = { 'A', 'B', 'C', 'D' };
-        try (final InputStream in = new BOMInputStream(createUtf8DataStream(data, false))) {
+        try (InputStream in = BOMInputStream.builder().setInputStream(createUtf8Input(data, false)).get()) {
             assertTrue(in.markSupported());
 
             in.mark(10);
@@ -372,28 +390,28 @@ public class BOMInputStreamTest {
     @Test
     public void testNoBoms() throws Exception {
         final byte[] data = { 'A', 'B', 'C' };
-        try {
-            (new BOMInputStream(createUtf8DataStream(data, true), false, (ByteOrderMark[])null)).close();
-            fail("Null BOMs, expected IllegalArgumentException");
-        } catch (final IllegalArgumentException e) {
-            // expected
-        }
-        try {
-            (new BOMInputStream(createUtf8DataStream(data, true), false, new ByteOrderMark[0])).close();
-            fail("Null BOMs, expected IllegalArgumentException");
-        } catch (final IllegalArgumentException e) {
-            // expected
+        assertThrows(IllegalArgumentException.class, () -> new BOMInputStream(createUtf8Input(data, true), false, (ByteOrderMark[]) null).close());
+        assertThrows(IllegalArgumentException.class, () -> new BOMInputStream(createUtf8Input(data, true), false, new ByteOrderMark[0]).close());
+        //
+        try (final BOMInputStream bomInputStream = BOMInputStream.builder()
+                .setInputStream(createUtf8Input(data, true))
+                .setInclude(true)
+                .setByteOrderMarks((ByteOrderMark[]) null)
+                .get()) {
+            assertEquals(BOMInputStream.Builder.getDefaultByteOrderMark(), bomInputStream.getBOM());
         }
+        assertThrows(IllegalArgumentException.class, () -> BOMInputStream.builder()
+                .setInputStream(createUtf8Input(data, true))
+                .setInclude(true)
+                .setByteOrderMarks()
+                .get()
+                .close());
     }
 
-
-
-
-
     @Test
     public void testReadEmpty() throws Exception {
         final byte[] data = {};
-        try (final BOMInputStream in = new BOMInputStream(createUtf8DataStream(data, false))) {
+        try (BOMInputStream in = BOMInputStream.builder().setInputStream(createUtf8Input(data, false)).get()) {
             assertEquals(-1, in.read());
             assertFalse(in.hasBOM(), "hasBOM()");
             assertFalse(in.hasBOM(ByteOrderMark.UTF_8), "hasBOM(UTF-8)");
@@ -404,7 +422,7 @@ public class BOMInputStreamTest {
     @Test
     public void testReadSmall() throws Exception {
         final byte[] data = { 'A', 'B' };
-        try (final BOMInputStream in = new BOMInputStream(createUtf8DataStream(data, false))) {
+        try (BOMInputStream in = BOMInputStream.builder().setInputStream(createUtf8Input(data, false)).get()) {
             assertEquals('A', in.read());
             assertEquals('B', in.read());
             assertEquals(-1, in.read());
@@ -427,7 +445,7 @@ public class BOMInputStreamTest {
     @Test
     public void testReadWithBOMInclude() throws Exception {
         final byte[] data = { 'A', 'B', 'C' };
-        try (final BOMInputStream in = new BOMInputStream(createUtf8DataStream(data, true), true)) {
+        try (BOMInputStream in = new BOMInputStream(createUtf8Input(data, true), true)) {
             assertEquals(0xEF, in.read());
             assertEquals(0xBB, in.read());
             assertEquals(0xBF, in.read());
@@ -444,8 +462,7 @@ public class BOMInputStreamTest {
     @Test
     public void testReadWithBOMUtf16Be() throws Exception {
         final byte[] data = "ABC".getBytes(StandardCharsets.UTF_16BE);
-        try (final BOMInputStream in = new BOMInputStream(createUtf16BeDataStream(data, true),
-                ByteOrderMark.UTF_16BE)) {
+        try (BOMInputStream in = new BOMInputStream(createUtf16BeDataStream(data, true), ByteOrderMark.UTF_16BE)) {
             assertEquals(0, in.read());
             assertEquals('A', in.read());
             assertEquals(0, in.read());
@@ -456,20 +473,14 @@ public class BOMInputStreamTest {
             assertTrue(in.hasBOM(), "hasBOM()");
             assertTrue(in.hasBOM(ByteOrderMark.UTF_16BE), "hasBOM(UTF-16BE)");
             assertEquals(ByteOrderMark.UTF_16BE, in.getBOM(), "getBOM");
-            try {
-                in.hasBOM(ByteOrderMark.UTF_16LE);
-                fail("Expected IllegalArgumentException");
-            } catch (final IllegalArgumentException e) {
-                // expected - not configured for UTF-16LE
-            }
+            assertThrows(IllegalArgumentException.class, () -> in.hasBOM(ByteOrderMark.UTF_16LE));
         }
     }
 
     @Test
     public void testReadWithBOMUtf16Le() throws Exception {
         final byte[] data = "ABC".getBytes(StandardCharsets.UTF_16LE);
-        try (final BOMInputStream in = new BOMInputStream(createUtf16LeDataStream(data, true),
-                ByteOrderMark.UTF_16LE)) {
+        try (BOMInputStream in = new BOMInputStream(createUtf16LeDataStream(data, true), ByteOrderMark.UTF_16LE)) {
             assertEquals('A', in.read());
             assertEquals(0, in.read());
             assertEquals('B', in.read());
@@ -480,12 +491,7 @@ public class BOMInputStreamTest {
             assertTrue(in.hasBOM(), "hasBOM()");
             assertTrue(in.hasBOM(ByteOrderMark.UTF_16LE), "hasBOM(UTF-16LE)");
             assertEquals(ByteOrderMark.UTF_16LE, in.getBOM(), "getBOM");
-            try {
-                in.hasBOM(ByteOrderMark.UTF_16BE);
-                fail("Expected IllegalArgumentException");
-            } catch (final IllegalArgumentException e) {
-                // expected - not configured for UTF-16BE
-            }
+            assertThrows(IllegalArgumentException.class, () -> in.hasBOM(ByteOrderMark.UTF_16BE));
         }
     }
 
@@ -493,7 +499,7 @@ public class BOMInputStreamTest {
     public void testReadWithBOMUtf32Be() throws Exception {
         assumeTrue(Charset.isSupported("UTF_32BE"));
         final byte[] data = "ABC".getBytes("UTF_32BE");
-        try (final BOMInputStream in = new BOMInputStream(createUtf32BeDataStream(data, true),
+        try (BOMInputStream in = new BOMInputStream(createUtf32BeDataStream(data, true),
                 ByteOrderMark.UTF_32BE)) {
             assertEquals(0, in.read());
             assertEquals(0, in.read());
@@ -511,12 +517,7 @@ public class BOMInputStreamTest {
             assertTrue(in.hasBOM(), "hasBOM()");
             assertTrue(in.hasBOM(ByteOrderMark.UTF_32BE), "hasBOM(UTF-32BE)");
             assertEquals(ByteOrderMark.UTF_32BE, in.getBOM(), "getBOM");
-            try {
-                in.hasBOM(ByteOrderMark.UTF_32LE);
-                fail("Expected IllegalArgumentException");
-            } catch (final IllegalArgumentException e) {
-                // expected - not configured for UTF-32LE
-            }
+            assertThrows(IllegalArgumentException.class, () -> in.hasBOM(ByteOrderMark.UTF_32LE));
         }
     }
 
@@ -524,7 +525,7 @@ public class BOMInputStreamTest {
     public void testReadWithBOMUtf32Le() throws Exception {
         assumeTrue(Charset.isSupported("UTF_32LE"));
         final byte[] data = "ABC".getBytes("UTF_32LE");
-        try (final BOMInputStream in = new BOMInputStream(createUtf32LeDataStream(data, true),
+        try (BOMInputStream in = new BOMInputStream(createUtf32LeDataStream(data, true),
                 ByteOrderMark.UTF_32LE)) {
             assertEquals('A', in.read());
             assertEquals(0, in.read());
@@ -542,19 +543,14 @@ public class BOMInputStreamTest {
             assertTrue(in.hasBOM(), "hasBOM()");
             assertTrue(in.hasBOM(ByteOrderMark.UTF_32LE), "hasBOM(UTF-32LE)");
             assertEquals(ByteOrderMark.UTF_32LE, in.getBOM(), "getBOM");
-            try {
-                in.hasBOM(ByteOrderMark.UTF_32BE);
-                fail("Expected IllegalArgumentException");
-            } catch (final IllegalArgumentException e) {
-                // expected - not configured for UTF-32BE
-            }
+            assertThrows(IllegalArgumentException.class, () -> in.hasBOM(ByteOrderMark.UTF_32BE));
         }
     }
 
     @Test
     public void testReadWithBOMUtf8() throws Exception {
         final byte[] data = "ABC".getBytes(StandardCharsets.UTF_8);
-        try (final BOMInputStream in = new BOMInputStream(createUtf8DataStream(data, true), ByteOrderMark.UTF_8)) {
+        try (BOMInputStream in = new BOMInputStream(createUtf8Input(data, true), ByteOrderMark.UTF_8)) {
             assertEquals('A', in.read());
             assertEquals('B', in.read());
             assertEquals('C', in.read());
@@ -562,19 +558,14 @@ public class BOMInputStreamTest {
             assertTrue(in.hasBOM(), "hasBOM()");
             assertTrue(in.hasBOM(ByteOrderMark.UTF_8), "hasBOM(UTF-8)");
             assertEquals(ByteOrderMark.UTF_8, in.getBOM(), "getBOM");
-            try {
-                in.hasBOM(ByteOrderMark.UTF_16BE);
-                fail("Expected IllegalArgumentException");
-            } catch (final IllegalArgumentException e) {
-                // expected - not configured for UTF-16BE
-            }
+            assertThrows(IllegalArgumentException.class, () -> in.hasBOM(ByteOrderMark.UTF_16BE));
         }
     }
 
     @Test
     public void testReadWithMultipleBOM() throws Exception {
         final byte[] data = { 'A', 'B', 'C' };
-        try (final BOMInputStream in = new BOMInputStream(createUtf8DataStream(data, true), ByteOrderMark.UTF_16BE,
+        try (BOMInputStream in = new BOMInputStream(createUtf8Input(data, true), ByteOrderMark.UTF_16BE,
                 ByteOrderMark.UTF_8)) {
             assertEquals('A', in.read());
             assertEquals('B', in.read());
@@ -590,7 +581,7 @@ public class BOMInputStreamTest {
     @Test
     public void testReadWithoutBOM() throws Exception {
         final byte[] data = { 'A', 'B', 'C' };
-        try (final BOMInputStream in = new BOMInputStream(createUtf8DataStream(data, false))) {
+        try (BOMInputStream in = BOMInputStream.builder().setInputStream(createUtf8Input(data, false)).get()) {
             assertEquals('A', in.read());
             assertEquals('B', in.read());
             assertEquals('C', in.read());
@@ -676,17 +667,17 @@ public class BOMInputStreamTest {
     @Test
     public void testReadXmlWithBOMUtf8() throws Exception {
         final byte[] data = "<?xml version=\"1.0\" encoding=\"UTF-8\"?><X/>".getBytes(StandardCharsets.UTF_8);
-        try (BOMInputStream in = new BOMInputStream(createUtf8DataStream(data, true))) {
+        try (BOMInputStream in = BOMInputStream.builder().setInputStream(createUtf8Input(data, true)).get()) {
             parseXml(in);
         }
-        parseXml(createUtf8DataStream(data, true));
+        parseXml(createUtf8Input(data, true));
     }
 
     @Test
     public void testReadXmlWithoutBOMUtf32Be() throws Exception {
         assumeTrue(jvmAndSaxBothSupportCharset("UTF_32BE"), "JVM and SAX need to support UTF_32BE for this");
         final byte[] data = "<?xml version=\"1.0\" encoding=\"UTF_32BE\"?><X/>".getBytes("UTF_32BE");
-        try (BOMInputStream in = new BOMInputStream(createUtf32BeDataStream(data, false))) {
+        try (BOMInputStream in = BOMInputStream.builder().setInputStream(createUtf8Input(data, false)).get()) {
             parseXml(in);
         }
         parseXml(createUtf32BeDataStream(data, false));
@@ -696,16 +687,34 @@ public class BOMInputStreamTest {
     public void testReadXmlWithoutBOMUtf32Le() throws Exception {
         assumeTrue(jvmAndSaxBothSupportCharset("UTF_32LE"), "JVM and SAX need to support UTF_32LE for this");
         final byte[] data = "<?xml version=\"1.0\" encoding=\"UTF-32LE\"?><X/>".getBytes("UTF_32LE");
-        try (BOMInputStream in = new BOMInputStream(createUtf32LeDataStream(data, false))) {
+        try (BOMInputStream in = BOMInputStream.builder().setInputStream(createUtf8Input(data, false)).get()) {
             parseXml(in);
         }
         parseXml(createUtf32BeDataStream(data, false));
     }
 
+    @Test
+    public void testSkipReturnValueWithBom() throws IOException {
+        final byte[] data = { (byte) 0x31, (byte) 0x32, (byte) 0x33 };
+        try (BOMInputStream is1 = BOMInputStream.builder().setInputStream(createUtf8Input(data, true)).get()) {
+            assertEquals(2, is1.skip(2));
+            assertEquals((byte) 0x33, is1.read());
+        }
+    }
+
+    @Test
+    public void testSkipReturnValueWithoutBom() throws IOException {
+        final byte[] data = { (byte) 0x31, (byte) 0x32, (byte) 0x33 };
+        try (BOMInputStream is2 = BOMInputStream.builder().setInputStream(createUtf8Input(data, false)).get()) {
+            assertEquals(2, is2.skip(2)); // IO-428
+            assertEquals((byte) 0x33, is2.read());
+        }
+    }
+
     @Test
     public void testSkipWithBOM() throws Exception {
         final byte[] data = { 'A', 'B', 'C', 'D' };
-        try (final InputStream in = new BOMInputStream(createUtf8DataStream(data, true))) {
+        try (InputStream in = BOMInputStream.builder().setInputStream(createUtf8Input(data, true)).get()) {
             in.skip(2L);
             assertEquals('C', in.read());
         }
@@ -714,35 +723,16 @@ public class BOMInputStreamTest {
     @Test
     public void testSkipWithoutBOM() throws Exception {
         final byte[] data = { 'A', 'B', 'C', 'D' };
-        try (final InputStream in = new BOMInputStream(createUtf8DataStream(data, false))) {
+        try (InputStream in = BOMInputStream.builder().setInputStream(createUtf8Input(data, false)).get()) {
             in.skip(2L);
             assertEquals('C', in.read());
         }
     }
 
-
-    @Test
-    public void skipReturnValueWithBom() throws IOException {
-        final byte[] baseData = { (byte) 0x31, (byte) 0x32, (byte) 0x33 };
-        try (final BOMInputStream is1 = new BOMInputStream(createUtf8DataStream(baseData, true))) {
-            assertEquals(2, is1.skip(2));
-            assertEquals((byte) 0x33, is1.read());
-        }
-    }
-
-    @Test
-    public void skipReturnValueWithoutBom() throws IOException {
-        final byte[] baseData = { (byte) 0x31, (byte) 0x32, (byte) 0x33 };
-        try (final BOMInputStream is2 = new BOMInputStream(createUtf8DataStream(baseData, false))) {
-            assertEquals(2, is2.skip(2)); // IO-428
-            assertEquals((byte) 0x33, is2.read());
-        }
-    }
-
     @Test
     public void testSmallBufferWithBOM() throws Exception {
         final byte[] data = { 'A', 'B', 'C' };
-        try (final InputStream in = new BOMInputStream(createUtf8DataStream(data, true))) {
+        try (InputStream in = BOMInputStream.builder().setInputStream(createUtf8Input(data, true)).get()) {
             final byte[] buf = new byte[1024];
             assertData(new byte[] { 'A', 'B' }, buf, in.read(buf, 0, 2));
             assertData(new byte[] { 'C' }, buf, in.read(buf, 0, 2));
@@ -752,7 +742,7 @@ public class BOMInputStreamTest {
     @Test
     public void testSmallBufferWithoutBOM() throws Exception {
         final byte[] data = { 'A', 'B', 'C' };
-        try (final InputStream in = new BOMInputStream(createUtf8DataStream(data, false))) {
+        try (InputStream in = BOMInputStream.builder().setInputStream(createUtf8Input(data, false)).get()) {
             final byte[] buf = new byte[1024];
             assertData(new byte[] { 'A', 'B' }, buf, in.read(buf, 0, 2));
             assertData(new byte[] { 'C' }, buf, in.read(buf, 0, 2));
@@ -762,7 +752,7 @@ public class BOMInputStreamTest {
     @Test
     // make sure that our support code works as expected
     public void testSupportCode() throws Exception {
-        try (final InputStream in = createUtf8DataStream(new byte[] { 'A', 'B' }, true)) {
+        try (InputStream in = createUtf8Input(new byte[] { 'A', 'B' }, true)) {
             final byte[] buf = new byte[1024];
             final int len = in.read(buf);
             assertEquals(5, len);
@@ -775,23 +765,4 @@ public class BOMInputStreamTest {
             assertData(new byte[] { (byte) 0xEF, (byte) 0xBB, (byte) 0xBF, 'A', 'B' }, buf, len);
         }
     }
-
-    private boolean jvmAndSaxBothSupportCharset(final String charSetName) throws ParserConfigurationException, SAXException, IOException {
-        return Charset.isSupported(charSetName) &&  doesSaxSupportCharacterSet(charSetName);
-    }
-
-    private boolean doesSaxSupportCharacterSet(final String charSetName) throws ParserConfigurationException, SAXException, IOException {
-        final byte[] data = ("<?xml version=\"1.0\" encoding=\"" + charSetName + "\"?><Z/>").getBytes(charSetName);
-        final DocumentBuilder documentBuilder = DocumentBuilderFactory.newInstance().newDocumentBuilder();
-        try {
-            final InputSource is = new InputSource(new ByteArrayInputStream(data));
-            is.setEncoding(charSetName);
-            documentBuilder.parse(is);
-        } catch (final SAXParseException e) {
-            if (e.getMessage().contains(charSetName)) {
-                return false;
-            }
-        }
-        return true;
-    }
 }
